from collections import deque, defaultdict


def bfs(guards, max_distances, graph):
    visited = set()
    queue = deque()

    for guard, distance in zip(guards, max_distances):
        queue.append((guard, 0))
        visited.add(guard)

    while queue:
        current, distance = queue.popleft()

        if distance < max(max_distances):
            for neighbor in graph[current]:
                if neighbor not in visited:
                    visited.add(neighbor)
                    queue.append((neighbor, distance + 1))

    return visited


def main():
    import sys
    input = sys.stdin.read
    data = input().splitlines()

    n, m, k = map(int, data[0].split())
    graph = defaultdict(list)

    for i in range(1, m + 1):
        a, b = map(int, data[i].split())
        graph[a].append(b)
        graph[b].append(a)

    guards = []
    max_distances = []

    for i in range(m + 1, m + k + 1):
        p, h = map(int, data[i].split())
        guards.append(p)
        max_distances.append(h)

    protected_stores = bfs(guards, max_distances, graph)

    protected_stores = sorted(protected_stores)

    print(len(protected_stores))
    print(" ".join(map(str, protected_stores)))


if __name__ == "__main__":
    main()