Python

from collections import deque, defaultdict


def bfs(start_points, max_distance, graph):
    visited = set()
    queue = deque(start_points)  # Начинаем с всех охранников
    for point in start_points:
        visited.add(point[0])  # Добавляем начальные склады в посещенные

    while queue:
        node, distance = queue.popleft()

        if distance < max_distance:
            for neighbor in graph[node]:
                if neighbor not in visited:
                    visited.add(neighbor)
                    queue.append((neighbor, distance + 1))

    return visited


def main():
    n, m, k = map(int, input().split())

    # Построение графа
    graph = defaultdict(list)

    for _ in range(m):
        a, b = map(int, input().split())
        graph[a].append(b)
        graph[b].append(a)

    start_points = []

    # Считывание охранников
    for _ in range(k):
        p, h = map(int, input().split())
        start_points.append((p, h))

    # Объединенный BFS
    protected_warehouses = set()

    for p, h in start_points:
        # Запускаем BFS для каждого охранника
        reachable_warehouses = bfs([(p, 0)], h, graph)
        protected_warehouses.update(reachable_warehouses)

    # Подготовка результата
    protected_warehouses = sorted(protected_warehouses)

    # Вывод результата
    print(len(protected_warehouses))
    if protected_warehouses:
        print(" ".join(map(str, protected_warehouses)))
    else:
        print()  # Пустая строка если нет охраняемых складов


if __name__ == "__main__":
    main()